@@ -289,23 +289,45 @@ llvm_middle_end_bind_parameters(struct draw_pt_middle_end *middle)
    unsigned i;
 
    for (i = 0; i < ARRAY_SIZE(llvm->jit_context.vs_constants); ++i) {
+      /*
+       * There could be a potential issue with rounding this up, as the
+       * shader expects 16-byte allocations, the fix is likely to move
+       * to LOAD intrinsic in the future and remove the vec4 constraint.
+       */
       int num_consts =
-         draw->pt.user.vs_constants_size[i] / (sizeof(float) * 4);
+         DIV_ROUND_UP(draw->pt.user.vs_constants_size[i], (sizeof(float) * 4));
       llvm->jit_context.vs_constants[i] = draw->pt.user.vs_constants[i];
       llvm->jit_context.num_vs_constants[i] = num_consts;
       if (num_consts == 0) {
          llvm->jit_context.vs_constants[i] = fake_const_buf;
       }
    }
+   for (i = 0; i < ARRAY_SIZE(llvm->jit_context.vs_ssbos); ++i) {
+      int num_ssbos = draw->pt.user.vs_ssbos_size[i];
+      llvm->jit_context.vs_ssbos[i] = draw->pt.user.vs_ssbos[i];
+      llvm->jit_context.num_vs_ssbos[i] = num_ssbos;
+      if (num_ssbos == 0) {
+         llvm->jit_context.vs_ssbos[i] = (const uint32_t *)fake_const_buf;
+      }
+   }
+
    for (i = 0; i < ARRAY_SIZE(llvm->gs_jit_context.constants); ++i) {
       int num_consts =
-         draw->pt.user.gs_constants_size[i] / (sizeof(float) * 4);
+         DIV_ROUND_UP(draw->pt.user.gs_constants_size[i], (sizeof(float) * 4));
       llvm->gs_jit_context.constants[i] = draw->pt.user.gs_constants[i];
       llvm->gs_jit_context.num_constants[i] = num_consts;
       if (num_consts == 0) {
          llvm->gs_jit_context.constants[i] = fake_const_buf;
       }
    }
+   for (i = 0; i < ARRAY_SIZE(llvm->gs_jit_context.ssbos); ++i) {
+      int num_ssbos = draw->pt.user.gs_ssbos_size[i];
+      llvm->gs_jit_context.ssbos[i] = draw->pt.user.gs_ssbos[i];
+      llvm->gs_jit_context.num_ssbos[i] = num_ssbos;
+      if (num_ssbos == 0) {
+         llvm->gs_jit_context.ssbos[i] = (const uint32_t *)fake_const_buf;
+      }
+   }
 
    llvm->jit_context.planes =
       (float (*)[DRAW_TOTAL_CLIP_PLANES][4]) draw->pt.user.planes[0];
@@ -355,9 +377,9 @@ llvm_pipeline_generic(struct draw_pt_middle_end *middle,
    struct llvm_middle_end *fpme = llvm_middle_end(middle);
    struct draw_context *draw = fpme->draw;
    struct draw_geometry_shader *gshader = draw->gs.geometry_shader;
-   struct draw_prim_info gs_prim_info;
+   struct draw_prim_info gs_prim_info[TGSI_MAX_VERTEX_STREAMS];
    struct draw_vertex_info llvm_vert_info;
-   struct draw_vertex_info gs_vert_info;
+   struct draw_vertex_info gs_vert_info[TGSI_MAX_VERTEX_STREAMS];
    struct draw_vertex_info *vert_info;
    struct draw_prim_info ia_prim_info;
    struct draw_vertex_info ia_vert_info;
@@ -368,6 +390,7 @@ llvm_pipeline_generic(struct draw_pt_middle_end *middle,
    unsigned start_or_maxelt, vid_base;
    const unsigned *elts;
 
+   assert(fetch_info->count > 0);
    llvm_vert_info.count = fetch_info->count;
    llvm_vert_info.vertex_size = fpme->vertex_size;
    llvm_vert_info.stride = fpme->vertex_size;
@@ -421,12 +444,21 @@ llvm_pipeline_generic(struct draw_pt_middle_end *middle,
                                vert_info,
                                prim_info,
                                &vshader->info,
-                               &gs_vert_info,
-                               &gs_prim_info);
+                               gs_vert_info,
+                               gs_prim_info);
 
       FREE(vert_info->verts);
-      vert_info = &gs_vert_info;
-      prim_info = &gs_prim_info;
+      vert_info = &gs_vert_info[0];
+      prim_info = &gs_prim_info[0];
+      /*
+       * pt emit can only handle ushort number of vertices (see
+       * render->allocate_vertices).
+       * vsplit guarantees there's never more than 4096, however GS can
+       * easily blow this up (by a factor of 256 (or even 1024) max).
+       */
+      if (vert_info->count > 65535) {
+         opt |= PT_PIPELINE;
+      }
    } else {
       if (draw_prim_assembler_is_required(draw, prim_info, vert_info)) {
          draw_prim_assembler_run(draw, prim_info, vert_info,
@@ -451,7 +483,7 @@ llvm_pipeline_generic(struct draw_pt_middle_end *middle,
    }
 
    /* stream output needs to be done before clipping */
-   draw_pt_so_emit( fpme->so_emit, vert_info, prim_info );
+   draw_pt_so_emit( fpme->so_emit, 1, vert_info, prim_info );
 
    draw_stats_clipper_primitives(draw, prim_info);
 
